"""Main module."""
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from typing import Any, Iterator, List, Optional, TypedDict, Unpack, cast

import requests

BASE_URL = "https://music.abcradio.net.au/api/v1/plays/search.json"


class ABCRadio:
    """API wrapper for accessing playlist history of various
    Australian Broadcasting Corporation radio channels

    """

    def __init__(self) -> None:
        self.available_stations: List[
            str
        ] = "jazz,dig,doublej,unearthed,country,triplej,classic,kidslisten".split(",")
        self.BASE_URL: str = BASE_URL
        self.latest_offset: int = (
            0  # keep track of state when iterating through a longer playlist of songs.
        )
        self.latest_search_parameters: Optional[RequestParams] = None

    def search(self, **params: Unpack[RequestParams]) -> "SearchResult":
        """Send request to abc radio API endpoint and create SearchResult instance

        Parameters
        ----------
        **params: RequestParams
            params["channel"]:str any of channels in self.available_stations
            params['startDate']: datetime
        """

        query_url = self.BASE_URL + self.construct_query_string(**params)
        r = requests.get(query_url)
        json_respose = r.json()
        result = SearchResult.from_json(json_input=json_respose)
        self.latest_offset = result.offset
        self.latest_search_parameters = cast(RequestParams, params)
        return result

    @staticmethod
    def construct_query_string(**params: Unpack[RequestParams]) -> str:
        """
        Construct query string to communicate with ABC radio API


        Returns
        _______
        str
            e.g."?from=2020-04-30T03:00:00.000000Z&station=triplej&offset=0&limit=10"
            internally this will return the keys order:'from','to','station',''offset','limit'
            although the ordering is not a requirement of the underlying web API
        """
        from_ = params.pop("from_", None)
        to = params.pop("to", None)
        station = params.pop("station", None)
        offset = params.pop("offset", None)
        limit = params.pop("limit", None)
        params_list = []
        if from_ is not None:
            params_list.append("from=" + from_.strftime("%Y-%m-%dT%H:%M:%S.%fZ"))
        if to is not None:
            params_list.append("to=" + to.strftime("%Y-%m-%dT%H:%M:%S.%fZ"))
        if station is not None:
            params_list.append("station=" + str(station))
        if offset is not None:
            params_list.append("offset=" + str(offset))
        if limit is not None:
            params_list.append("limit=" + str(limit))

        if len(params_list) >= 1:
            return "?" + "&".join(params_list)
        else:
            return ""

    def continuous_search(
        self, **params: Unpack[RequestParams]
    ) -> Iterator[SearchResult]:

        initial_search = self.search(**params)
        yield initial_search
        total = initial_search.total
        offset = initial_search.offset
        limit = initial_search.limit
        while offset + limit < total:
            offset = offset + limit
            params["offset"] = offset
            yield self.search(**params)


class RequestParams(TypedDict, total=False):
    """
    **kwarg arguments to be used when searching in the ABC web api

    Parameters
    ----------
    from_: datetime
        The earliest data starts from "2014-04-30T03:00:04+00:00"

    to: datetime
        to value should be greater than from_

    limit: int
        number of results to display in a single request, effective limit is 100

    offset: int
        index at which to start returning results, you can iterate through all
        radio plays by updating this value. See continuous_search for an example
        implementation.
    station: str
       any one of: "jazz,dig,doublej,unearthed,country,triplej,classic,kidslisten"

    """

    from_: datetime
    to: datetime
    limit: int
    offset: int
    station: str


@dataclass
class RadioSong:
    """
    Dataclass for each entity returned from ABCradio.search,
    A RadioSong is a Song played at a specific time on a specific channel.


    Attributes
    ----------
    played_time: datetime
        original playtime, including timezone information

    channel: str
        Name of channel in which song was played
        e.g. "triplej","doublej","classic","jazz","unearthed","country"

    song: Song
        contains metadata of the song including artist and album details

    """

    played_time: datetime
    channel: str
    song: Song

    @classmethod
    def from_json(cls, json_input: dict[str, Any]) -> RadioSong:
        """
        Create RadioSong instance based on json_input

        Parameters
        ----------
        json_input : dict[str,Any]
            In the format of:
                {"entity":"Play",
                 "arid":"...",
                 "played_time":"2020-01-01T12:00:00+00:00"
                 "service_id":"triplej"
                 "recording":...,   # Song details (see Song class for more details)
                 "release": ...     # Album details (see Album class for more details)
                 }

        Returns
        _______
        RadioSong
        """
        song = Song.from_json(json_input)
        return cls(
            played_time=datetime.fromisoformat(json_input["played_time"]),
            channel=json_input["service_id"],
            song=song,
        )


@dataclass
class SearchResult:
    """
    Dataclass returned from ABCRadio.search
    """

    total: int
    offset: int
    limit: int
    radio_songs: List["RadioSong"]

    @classmethod
    def from_json(cls, json_input: dict[str, Any]) -> SearchResult:

        radio_songs = []
        for radio_song in json_input["items"]:
            radio_songs.append(RadioSong.from_json(radio_song))
        return cls(
            total=json_input["total"],
            offset=json_input["offset"],
            limit=json_input["limit"],
            radio_songs=radio_songs,
        )


@dataclass
class Song:
    """
    Dataclass to represent a song

    """

    title: str
    duration: int
    artists: List["Artist"]
    album: Optional[Album]
    url: Optional[str]

    @classmethod
    def from_json(cls, json_input: dict[str, Any]) -> Song:
        """
        Create Song instance based on json_input

        Parameters
        ----------
        json_input : dict[str,Any]
            In the format of:
                {"entity":"Play",
                 "arid":"...",
                 "played_time":"2020-01-01T12:00:00+00:00",
                 "service_id":"triplej",
                 "recording":...,   # Song details
                 "release": ...     # Album details
                 }


        Returns
        _______
        Song
        """
        try:
            # Occassionally release information is not present or only exists
            # under recordings.releases json key
            json_release = (
                json_input["release"]
                if json_input["release"]
                else json_input["recording"]["releases"][0]
            )
            album = Album.from_json(json_release)
            artists = []
            for artist in json_release["artists"]:
                artists.append(Artist.from_json(artist))
        except (KeyError, IndexError, TypeError):
            artists = []
            album = None

        url = Song.get_url(json_input)
        return cls(
            title=json_input["recording"]["title"],
            duration=json_input["recording"]["duration"],
            artists=artists,
            album=album,
            url=url,
        )

    @staticmethod
    def get_url(json_input):
        if len(json_input["recording"]["links"]) >= 1:
            return json_input["recording"]["links"][0]["url"]
        else:
            return None


@dataclass
class Artist:
    """Dataclass to represent Artists"""

    url: Optional[
        str
    ]  # http://musicbrainz.org/ws/2/artist/5b11f4ce-a62d-471e-81fc-a69a8278c7da\?inc\=aliases
    name: str
    is_australian: bool

    @classmethod
    def from_json(cls, json_input: dict[str, Any]) -> Artist:
        is_australian = bool(json_input["is_australian"])
        if len(json_input["links"]) >= 1:
            url = json_input["links"][0]["url"]
        else:
            url = None
        return cls(url=url, name=json_input["name"], is_australian=is_australian)


@dataclass
class Album:
    url: Optional[str]
    title: str
    artwork: Optional[Artwork]
    release_year: Optional[int]

    @classmethod
    def from_json(cls, json_input: dict[str, Any]) -> Album:
        try:
            artwork = Artwork.from_json(json_input["artwork"][0])
        except IndexError:
            artwork = None

        return cls(
            url=Album.get_url(json_input),
            title=json_input["title"],
            release_year=int(json_input["release_year"])
            if json_input["release_year"]
            else None,
            artwork=artwork,
        )

    @staticmethod
    def get_url(json_input):
        if len(json_input["links"]) >= 1:
            return json_input["links"][0]["url"]
        else:
            return None


@dataclass
class Artwork:
    url: str
    type: str
    sizes: List[ArtworkSize]

    @classmethod
    def from_json(cls, json_input: dict[str, Any]) -> Artwork:

        sizes: List[ArtworkSize] = []
        for size in json_input["sizes"]:
            sizes.append(ArtworkSize.from_json(size))
        return Artwork(url=json_input["url"], type=json_input["type"], sizes=sizes)


@dataclass
class ArtworkSize:
    url: str
    width: int
    height: int
    aspect_ratio: str

    @classmethod
    def from_json(cls, json_input: dict[str, Any]) -> ArtworkSize:
        return cls(
            url=json_input["url"],
            width=json_input["width"],
            height=json_input["height"],
            aspect_ratio=json_input["aspect_ratio"],
        )

    @property
    def aspect_ratio_float(self) -> float:
        width_ratio, height_ratio = self.aspect_ratio.split("x")
        return int(width_ratio) / int(height_ratio)



#!/usr/bin/env python

"""Tests for `abc_radio_wrapper` package."""


import unittest
import os
import json
from typing import List
from datetime import datetime

from abc_radio_wrapper import abc_radio_wrapper


class TestAbc_radio_wrapper(unittest.TestCase):
    """Tests for `abc_radio_wrapper` package."""

    def setUp(self):
        """Set up test fixtures. Load json search result from file"""
        print(os.getcwd())
        TESTDATA_FILENAME = os.path.join(
            os.path.dirname(__file__), "search_result.json"
        )
        with open(TESTDATA_FILENAME) as f:
            self.json_search_result: dict = json.load(f)

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def test_000_test_file_loaded(self):
        """Test setUp file is loaded"""
        self.assertEqual(self.json_search_result["total"], 142)

    def test_001_Song_object_creation(self):
        """Test Song instance can be created from json_input"""

        album = abc_radio_wrapper.Album.from_json(
            self.json_search_result["items"][0]["release"]
        )
        artists = []
        for artist in self.json_search_result["items"][0]["release"]["artists"]:
            artists.append(abc_radio_wrapper.Artist.from_json(artist))

        expected = abc_radio_wrapper.Song(
            title="Blue in Green",
            duration=277,
            url="http://musicbrainz.org/recording/11328517-1cce-4981-888b-f98cdeb0d7aa",
            artists=artists,
            album=album,
        )

        result = abc_radio_wrapper.Song.from_json(self.json_search_result["items"][0])
        self.assertEqual(expected, result)

    def test_002_test_Artist_object_creation(self):
        """Test Artist instance can be created from json_input"""
        result = abc_radio_wrapper.Artist.from_json(
            self.json_search_result["items"][0]["release"]["artists"][0]
        )
        expected = abc_radio_wrapper.Artist(
            name="Jim Snidero",
            url="http://musicbrainz.org/artist/20638241-3b98-461a-9677-8cb039489326",
            is_australian=False,
        )
        self.assertEqual(expected, result)

    def test_003_test_Album_object_creation(self):
        """Test Album instance can be created from json_input"""
        result = abc_radio_wrapper.Album.from_json(
            self.json_search_result["items"][0]["release"]
        )

        artwork = abc_radio_wrapper.Artwork.from_json(
            self.json_search_result["items"][0]["release"]["artwork"][0]
        )
        expected = abc_radio_wrapper.Album(
            url=None, title="MD66", release_year=2016, artwork=artwork
        )

        self.assertEqual(expected, result)

    def test_004_test_ArtworkSize_object_creation(self):
        """Test ArtworkSize instance can be created from json_input"""
        result = abc_radio_wrapper.ArtworkSize.from_json(
            self.json_search_result["items"][0]["release"]["artwork"][0]["sizes"][0]
        )
        expected = abc_radio_wrapper.ArtworkSize(
            url="https://resize.abcradio.net.au/Mv2oBeipagYezPEjutlReBRUE2I=/100x100/center/middle/"
            "http%3A%2F%2Fabc-dn-mapi-production.s3.ap-southeast-2.amazonaws.com%2Frelease%2F"
            "miBgqJJw9w%2Fecd96b225f7e933003fd2529df24d5bd.jpg",
            width=100,
            height=100,
            aspect_ratio="1x1",
        )
        self.assertEqual(expected, result)

    def test_005_test_Artwork_object_creation(self):
        """Test Artwork instance can be created from json_input"""
        result = abc_radio_wrapper.Artwork.from_json(
            self.json_search_result["items"][0]["release"]["artwork"][0]
        )
        sizes: List[abc_radio_wrapper.ArtworkSize] = []
        for size in self.json_search_result["items"][0]["release"]["artwork"][0][
            "sizes"
        ]:
            sizes.append(abc_radio_wrapper.ArtworkSize.from_json(size))

        expected = abc_radio_wrapper.Artwork(
            url="http://abc-dn-mapi-production.s3.ap-southeast-2.amazonaws.com"
            "/release/miBgqJJw9w/ecd96b225f7e933003fd2529df24d5bd.jpg",
            type="cover",
            sizes=sizes,
        )
        self.assertEqual(expected, result)

    def test_006_test_RadioSong_object_creation(self):
        """Test RadioSong instance can be created from json_input"""
        expected = False
        played_time = datetime.fromisoformat("2020-04-30T04:15:49+00:00")
        song = abc_radio_wrapper.Song.from_json(self.json_search_result["items"][0])
        expected = abc_radio_wrapper.RadioSong(
            played_time=played_time, channel="jazz", song=song
        )
        result = abc_radio_wrapper.RadioSong.from_json(
            self.json_search_result["items"][0]
        )
        self.assertEqual(expected, result)

    def test_007_test_SearchResult_object_creation(self):
        """Test SearchResult object can be created from search json response"""

        radio_songs = []
        for radio_song in self.json_search_result["items"]:
            radio_songs.append(abc_radio_wrapper.RadioSong.from_json(radio_song))
        expected = abc_radio_wrapper.SearchResult(
            total=142, offset=0, limit=10, radio_songs=radio_songs
        )
        result = abc_radio_wrapper.SearchResult.from_json(self.json_search_result)
        self.assertEqual(expected, result)

    def test_008_test_Search(self):
        """Test Search with no parameters works when interacting
        with the abc radio API endpoint
        """

        ABCWrapper = abc_radio_wrapper.ABCRadio()

        result = ABCWrapper.search()
        self.assertEqual(result.offset, 0)
        self.assertEqual(result.limit, 10)

    def test_009_create_query_string(self):

        expected = (
            "?from=2020-04-30T03:00:00.000000Z&"
            "to=2020-04-30T03:16:00.000000Z&station=jazz&offset=0&limit=10"
        )

        startDate: datetime = datetime.fromisoformat("2020-04-30T03:00:00+00:00")
        endDate: datetime = datetime.fromisoformat("2020-04-30T03:16:00+00:00")

        result: str = abc_radio_wrapper.ABCRadio.construct_query_string(
            from_=startDate, to=endDate, station="jazz", offset=0, limit=10
        )

        self.assertEqual(expected, result)

    def test_010_test_Search_parameters(self):
        """
        Test Search parameters such as 'station',
        'from' and that they affect search results
        """
        ABCRadio = abc_radio_wrapper.ABCRadio()
        startDate = datetime.fromisoformat("2020-04-30T03:00:00+00:00")
        endDate = datetime.fromisoformat("2020-04-30T04:16:00+00:00")
        result = ABCRadio.search(from_=startDate, to=endDate)
        expected = abc_radio_wrapper.SearchResult.from_json(self.json_search_result)

        self.assertEqual(expected, result)

    def test_011_test_Search_offset(self):
        """test that offset is applied when added as parameter"""
        ABCWrapper = abc_radio_wrapper.ABCRadio()

        result = ABCWrapper.search(offset=10)
        self.assertEqual(result.offset, 10)
        self.assertEqual(result.limit, 10)

    def test_012_test_Search_iteration(self):
        """
        test that continuous_search is able to search iteratively
        while updating the offset parameter.
        """
        ABCWrapper = abc_radio_wrapper.ABCRadio()

        startDate: datetime = datetime.fromisoformat("2020-04-30T03:00:00+00:00")
        endDate: datetime = datetime.fromisoformat("2020-04-30T03:15:00+00:00")

        i = 0
        offsets = []
        for searchresult in ABCWrapper.continuous_search(
            from_=startDate, to=endDate, limit=10
        ):
            i += 1
            offsets.append(searchresult.offset)
            total = searchresult.total

        self.assertTrue(i > 2)
        self.assertEqual([0, 10, 20, 30], offsets)
        self.assertEqual(31, total)



#!/usr/bin/env python3

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "requests>=2.27.1"
]

test_requirements = [
        "mypy>=0.991",
        "mypy-extensions>=0.4.3",
        "types-requests>=2.28.11.8",
        "types-urllib3>=1.26.25.4",
        "typing_extensions>=4.4.0",
        "requests>=2.27.1",
        "black>=22.12.0",
        "flake8>==6.0.0",
        "Sphinx>=5.3.0",
        "sphinx-rtd-theme>=1.1.1"
]

setup(
    author="Matthew Burke",
    author_email='mperoburke@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="API wrapper library for the song history of abc radio channels",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='abc_radio_wrapper',
    name='abc_radio_wrapper',
    packages=find_packages(include=['abc_radio_wrapper', 'abc_radio_wrapper.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/MatthewBurke1995/ABC-Radio-Wrapper',
    version='0.2.0',
    zip_safe=False,
)
